package com.middlemind.Eds3App;

import com.middlemind.Eds3.Eds3Client;
import com.middlemind.Eds3.Eds3Server;
import com.middlemind.Eds3.GameRoom;
import com.middlemind.Eds3.Utils;
import com.middlemind.Eds3.bhw.Eds3ServerBhw;
import com.middlemind.Eds3.mock.MockGenDungeon;
import com.middlemind.Eds3.unity.GameObjectWrapperDefault;
import java.util.ArrayList;

/**
 *
 * @author Victor G. Brusca, Middlemind Games
 */
public class Eds3RunApp {

    public static int MAX_CLIENTS = 6;
    public static ArrayList<Eds3Client> CLIENTS = new ArrayList();

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            if(args != null) {
                Utils.wr("Found " + args.length + " args.");
            }
            
            if (args != null && args.length >= 1) {
                Eds3Server eds3server = null;
                Eds3Client eds3client = null;
                Eds3ServerBhw eds3serverBhw = null;
                
                if (args[0].toLowerCase().equals("server")) {
                    eds3server = new Eds3Server();

                    if (args.length >= 2) {
                        eds3server.SetLocalIpAddress(args[1]);
                    }

                    if (args.length >= 3) {
                        eds3server.SetPort(args[2]);
                    }
                    
                    boolean tmp = false;
                    if (args.length >= 4) {
                        tmp = Boolean.parseBoolean(args[3]);
                        Utils.LOGGING_ON = tmp;
                        eds3server.SetDebugOn(tmp);
                    }
                    
                    if (args.length >= 5) {                        
                        Utils.VERBOSE = tmp = Boolean.parseBoolean(args[4]);
                    }
                    
                    GameRoom.PLAYER_THRESHOLD_INACTIVE_COMM_MS = 3000;
                    eds3server.Start();

                    String inp = "";
                    System.out.println("Enter 'q' to exit.");
                    while (inp != null && !inp.equals("q")) {
                        System.out.print("Entry: ");
                        inp = System.console().readLine().toLowerCase().trim();
                        Utils.wr("Found cmd: " + inp);
                        if (!inp.equals("q")) {
                            eds3server.ProcessCmd(inp);
                            Thread.sleep(2000);
                        }
                    }

                    Utils.wr("Out of loop...");
                    eds3server.Stop();
                    Utils.wr("Exiting...");
                    return;

                }else if (args[0].toLowerCase().equals("bhw_server")) {
                    eds3serverBhw = new Eds3ServerBhw();

                    if (args.length >= 2) {
                        eds3serverBhw.SetLocalIpAddress(args[1]);
                    }

                    if (args.length >= 3) {
                        eds3serverBhw.SetPort(args[2]);
                    }
                    
                    boolean tmp = false;
                    if (args.length >= 4) {
                        tmp = Boolean.parseBoolean(args[3]);
                        Utils.LOGGING_ON = tmp;
                        eds3serverBhw.SetDebugOn(tmp);
                    }
                    
                    if (args.length >= 5) {
                        Utils.VERBOSE = Boolean.parseBoolean(args[4]);
                    }                    
                    
                    GameRoom.PLAYER_THRESHOLD_INACTIVE_COMM_MS = 3000;
                    eds3serverBhw.Start();

                    String inp = "";
                    System.out.println("Enter 'q' to exit.");
                    while (inp != null && !inp.equals("q")) {
                        System.out.print("Entry: ");
                        inp = System.console().readLine().toLowerCase().trim();
                        Utils.wr("Found cmd: " + inp);
                        if (!inp.equals("q")) {
                            eds3serverBhw.ProcessCmd(inp);
                            Thread.sleep(2000);
                        }
                    }

                    Utils.wr("Out of loop...");
                    eds3serverBhw.Stop();
                    Utils.wr("Exiting...");
                    return;
                    
                } else if (args[0].toLowerCase().equals("client")) {
                    Eds3Client.CONSOLIDATE_LOGS_ON = false;
                    eds3client = new Eds3Client(Eds3Server.GetLocalIPAddressStat(), 49999);

                    if (args.length >= 2) {
                        eds3client.SetLocalIpAddress(args[1]);
                    }
                    
                    if (args.length >= 3) {
                        eds3client.SetPort(args[2]);
                    }

                    Eds3Client.NOT_UNITY = true;
                    eds3client.Start();

                    String inp = "";
                    System.out.println("Enter 'q' to exit.");
                    while (inp != null && !inp.equals("q")) {
                        System.out.print("Entry: ");
                        inp = System.console().readLine().toLowerCase().trim();
                        Utils.wr("Found cmd: " + inp);
                        if (!inp.equals("q")) {
                            eds3client.ProcessCmd(inp);
                            Thread.sleep(2000);
                        }
                    }

                    eds3client.Stop();
                    Utils.wr("Exiting...");
                    return;

                } else if (args[0].toLowerCase().equals("clients")) {
                    CLIENTS.clear();
                   
                    Utils.wr("Starting " + MAX_CLIENTS + " new clients.");
                    Eds3Client tmpEds3client = null;
                    Eds3Client.CONSOLIDATE_LOGS_ON = false;
                    Eds3Client.NOT_UNITY = true;
                    for (int i = 0; i < MAX_CLIENTS; i++) {                        
                        if (args.length >= 2) {
                            tmpEds3client = new Eds3Client(args[1], 49999);
                        } else {
                            tmpEds3client = new Eds3Client(Eds3Server.GetLocalIPAddressStat(), 49999);
                        }                        
                        
                        if (args.length >= 3) {
                            tmpEds3client.SetPort(args[2]);
                        }
            
                        CLIENTS.add(tmpEds3client);
                        tmpEds3client.Start();
                    }

                    String inp = "";
                    while (inp != null && !inp.equals("q")) {
                        System.out.print("Entry: ");
                        inp = System.console().readLine().toLowerCase();
                        Thread.sleep(2000);
                    }

                    Utils.wr("Found stop command.");
                    Eds3Client eds3c;
                    for (int i = 0; i < CLIENTS.size(); i++) {
                        eds3c = CLIENTS.get(i);
                        Utils.wr("Stopping client: " + eds3c.GetClientId() + " with SpawnId/PlayerId: " + eds3c.GetSpawnId());
                        eds3c.Stop();

                    }
                    Utils.wr("Exiting...");
                    return;

                } else if (args[0].toLowerCase().equals("sertest")) {
                    MockGenDungeon.SLEEP = false;
                    MockGenDungeon mgd1 = new MockGenDungeon(100);
                    Utils.wr("Generating...");
                    long st = System.currentTimeMillis();
                    mgd1.GenerateObjects();
                    mgd1.GenerateSpawnObjects();
                    long sp = System.currentTimeMillis();
                    Utils.wr("Generation Time MS: " + (sp - st));

                    Utils.wr("Serializing...");
                    st = System.currentTimeMillis();
                    byte[] objs = mgd1.SerializeNetworkGameObjects();
                    sp = System.currentTimeMillis();
                    Utils.wr("Serialization Time MS: " + (sp - st));
                    Utils.wr("Found " + objs.length + " bytes of serialized data.");
                    Utils.wr("Expected " + (GameObjectWrapperDefault.TOTAL_LENGTH * mgd1.GetNetworkGameObjectsCount()) + " bytes of serialized data.");

                    Utils.wr("Deserializing...");
                    st = System.currentTimeMillis();
                    MockGenDungeon mgd2 = new MockGenDungeon(100);
                    mgd2.DeserializeNetworkGameObjects(objs);
                    sp = System.currentTimeMillis();
                    Utils.wr("Deserialization Time MS: " + (sp - st));

                    boolean res = mgd1.NetworkGameObjectsEquals(mgd2);
                    Utils.wr("Serialization Test Success: " + res);
                    return;

                } else if (args[0].toLowerCase().equals("serstats")) {
                    long totalGenTimeMs = 0;
                    long totalSerTimeMs = 0;
                    long totalDserTimeMs = 0;
                    int cnt = 10000;

                    long st;
                    long sp;
                    byte[] objs;
                    boolean res;
                    MockGenDungeon mgd1;
                    MockGenDungeon mgd2;

                    Utils.wr("Running " + cnt + " tests...");
                    Utils.LOGGING_ON = false;
                    Utils.VERBOSE = false;
                    MockGenDungeon.SLEEP = false;

                    for (int i = 0; i < cnt; i++) {

                        mgd1 = new MockGenDungeon(100);
                        st = System.currentTimeMillis();
                        mgd1.GenerateObjects();
                        mgd1.GenerateSpawnObjects();
                        sp = System.currentTimeMillis();
                        totalGenTimeMs += (sp - st);

                        st = System.currentTimeMillis();
                        objs = mgd1.SerializeNetworkGameObjects();
                        sp = System.currentTimeMillis();
                        totalSerTimeMs += (sp - st);

                        mgd2 = new MockGenDungeon(100);
                        st = System.currentTimeMillis();
                        mgd2.DeserializeNetworkGameObjects(objs);
                        sp = System.currentTimeMillis();
                        totalDserTimeMs += (sp - st);

                        res = mgd1.NetworkGameObjectsEquals(mgd2);
                        if (!res) {
                            Utils.LOGGING_ON = true;
                            Utils.wr("Comparison failure, iteration: " + i);
                            break;
                        }

                        mgd1.RandomlyAdjustObjects();
                    }

                    Utils.LOGGING_ON = true;
                    double avgGenTimeMs = (double) totalGenTimeMs / cnt;
                    double avgSerTimeMs = (double) totalSerTimeMs / cnt;
                    double avgDserTimeMs = (double) totalDserTimeMs / cnt;

                    Utils.wr("TotalGenTimeMs: " + totalGenTimeMs);
                    Utils.wr("TotalSerTimeMs: " + totalSerTimeMs);
                    Utils.wr("TotalDserTimeMs: " + totalDserTimeMs);

                    Utils.wr("AvgGenTimeMs: " + avgGenTimeMs);
                    Utils.wr("AvgSerTimeMs: " + avgSerTimeMs);
                    Utils.wr("AvgDserTimeMs: " + avgDserTimeMs);
                    Utils.wr("60 FPS = 1 frame every " + (1000 / 60) + " ms.");
                    return;
                }
            }
        } catch (Exception e) {
            Utils.wr(e.getMessage());
            e.printStackTrace();
        }
    }
}
